#!/bin/sh

original_tty_state=$(stty -g)
dcsOnOff=0      # Turns the DCS on (off) before (after) running all the scans
quietMode=0     # Trims the output to the DB-related info + the scan results
createConfig=0  # Creates / overwrites Yarr config files based on config.json
justConfigure=0 # Update config and exit
skipCfgCheck=0  # Skips checking for cfg updates (saving time)
dcsMonitor=0    # 
envMonitor=0    # 
PDOpt=0         # Downloads chip config files from Production DB

# A few colors for echoing
function darkblue(){
    echo "\033[0;34m$1\033[0m"
}
function green(){
    echo "\033[0;32m$1\033[0m"
}
function red(){
    echo "\033[0;31m$1\033[0m"
}
function yellow(){
    echo "\033[0;33m$1\033[0m"
}

function wrapAndExit(){
    echo "[ info ][so] Wrapping and exiting after getting an error"
    
    echo -e "\n-------------------------"

    echo "[ info ][so] Recording total elapsed time..."
    elp=$(echo "$(date +%s.%N) - $START_SO" | bc)
    echo -ne "total_so     $elp\n" >> $TIMEFILE
    

    echo "[ info ][so] Reverting the terminal to its original state..."
    stty $original_tty_state

    if [[ $dcsMonitor == 1 ]]; then
        echo "[ info ][so] waiting for the dcs monitor script to finish..."
        echo "0" > $SOBASE/.counters/dcsStat
        waitForItToFinish "read_dcs_backgr"
    fi
     
    echo "[ info ][so] -- done -- "
    exit 1
}

function sigintStop(){
    echo -e "\n[ info ][so] Stoped by SIGINT"
    wrapAndExit
}

trap 'sigintStop' 2

function waitForItToFinish(){
    while [ "$(pgrep -x "$1")" != "" ]
    do
        sleep 0.1
    done
}

function usage(){
    cat <<EOF
$0
Help:
  -m [str]   : Serial number of the module
  -W         : upload Yarr's results to localDB (scanConsole's -W)
  -o         : turn on / off the output of the PS before and after all the scans
  -e         : Check if T and H are within a setting range before running each scan
  -d         : Monitor the PS and upload the DCS data to influxDB.
  -t         : Move DCS data from InfluxDB to LocalDB after each scan.
  -c         : Create / override Yarr's config files for your chip.
  -s         : Don't look for updates in the config (saves time)
  -j         : Just update the configs, and exit before running any scan
  -q         : "quiet" mode (cleaner output).
  -P         : Download config files of ITk Pixel Modules from the ITkPD.(Not yet implemented.)
EOF
    exit 1
}

# Check now if jq exists to prevent further weird-looking jq-related errors
if ! command -v jq &> /dev/null
then
    echo "[ error ][so] command \"jq\" not found! Please install it before proceeding."
    exit
fi

while getopts m:q:docstejWQP OPT
do
    case $OPT in
        m) module_id=$OPTARG ;;
        q) quietMode=1 ;;
        t) syncOption="-t" ;;
        e) envMonitor=1 ;;
        o) dcsOnOff=1 ;;
        d) dcsMonitor=1 ;;
        c) createConfig=1 ;;
        s) skipCfgCheck=1 ;;
        j) justConfigure=1 ;;
        W) localdbOpt="-W" ;;
	P) PDOpt=1 ;;
        Q) qcOpt="-Q" ;;
        *) usage ;;
    esac
done

echo "--------------------------------"

# Two important constants here
export SOBASE=$(cd $(dirname $0); pwd)
INDEXJSON="$SOBASE/configs/so_index.json"

if [ ! -f $INDEXJSON ];then
    echo "[ error ][so] Not found file \"$INDEXJSON\""
    echo "[ error ][so] Put the file back there and try again."
    usage
    exit
fi

if [[ $syncOption == "-t" ]] && [[ $localdbOpt == "" ]]; then
    echo "[ error ][so] To transfer DCS data to localDB, first you have to have"
    echo "              the module information in localDB! Option \"-t\" requires also \"-W\"."
    exit
fi

# Config files used here and there
MODULECFG="$SOBASE/`jq -j '.module_config' $INDEXJSON`"
INFLUXDBENVCFG="$SOBASE/`jq -j '.influxdb_config_env' $INDEXJSON`"
export INFLUXDBDCSCFG="$SOBASE/`jq -j '.influxdb_config_dcs' $INDEXJSON`"
export SCANCFG="$SOBASE/`jq -j '.rd53a_scans_config' $INDEXJSON`"
export DBCOMCFG="$SOBASE/`jq -j '.dbs_communication_config' $INDEXJSON`"
export PSCFG="$SOBASE/`jq -j '.powersupply_config' $INDEXJSON`"
export LDBUSRCFG="$SOBASE/`jq -j '.localdb_user_config' $INDEXJSON`"
export LDBSITCFG="$SOBASE/`jq -j '.localdb_site_config' $INDEXJSON`"

export YARRBASE=`jq -j '.YARR_directory' $SCANCFG`


if [ -z $module_id ];then
    echo "Please specify a Module ID (option '-m'). "
    usage
fi

if [ ! -d "$YARRBASE" ]; then
    echo -e "[ error ][so] Please specify a valid path to Yarr under \n" \
            "             \".YARR_directory\" in \"$SCANCFG\""
    exit 1
fi


mkdir -p $SOBASE/.counters
touch $SOBASE/.counters/runNumber $SOBASE/.counters/today
if ! [[ "$(<$SOBASE/.counters/runNumber)" =~ ^[0-9]+$ ]]; then
    echo "0" > $SOBASE/.counters/runNumber
fi
if [[ $(date "+%y%m%d") -gt $(<$SOBASE/.counters/today) ]]; then
    echo $(date "+%y%m%d") > $SOBASE/.counters/today
    echo "0" > $SOBASE/.counters/runNumber
fi


echo "$(($(<$SOBASE/.counters/runNumber)+1))" > $SOBASE/.counters/runNumber

export SO_RUNNUMBER="$(<$SOBASE/.counters/today)_$(printf "%03d" $(<$SOBASE/.counters/runNumber))"

# data folder
mkdir -p $SOBASE/data/$SO_RUNNUMBER

# time data
touch $SOBASE/data/$SO_RUNNUMBER/time.dat
export TIMEFILE="$SOBASE/data/$SO_RUNNUMBER/time.dat"
echo -ne "# what     Elapsed (s)\n" >> $TIMEFILE

# start measuring time
START_SO=$(date +%s.%N)


if [[ $dcsOnOff ]] || [[ $dcsMonitor ]]
then
    lrLibPath=`jq -j '.LR_lib_path' $PSCFG`
    export PYTHONPATH=$lrLibPath:$PYTHONPATH
fi


if  [[ $dcsMonitor == 1 ]]
then
    chn=""

    nChannels="$(jq -j '.channels_to_be_used_by_the_scanoperator | length' $PSCFG)"
    for (( i=0; i<$nChannels; i++ ))
    do
        chName="$(jq -j ".channels_to_be_used_by_the_scanoperator[$i].name" $PSCFG)"
        chn+="$chName,"
    done
    chn="${chn::-1}"
    
    # Perform a first DCS Measurement to check for possible authentication
    while true; do 
        python3 $SOBASE/libDCS/psOperator.py -e $PSCFG -c $chn -d $INFLUXDBDCSCFG measure
        rv=$?
        if [ $rv -eq 0 ]; then 
            # No authentication / No errors
            break

        elif [ $rv -eq 1 ]; then 
            # Error othen than authentication failure
            exit 1; 

        elif [ $rv -eq 2 ]; then

            # Authentication failure
            echo -n "Password: " 
            read -s pwd
            export INFLUXDBPWD=$pwd
            echo ""
        fi
    done
fi

if [[ "$syncOption" == "" ]] && [[ "$localdbOpt" != "" ]]
then
    syncScript=$SOBASE/data/$SO_RUNNUMBER/moveDCStoLocalDB.sh
    touch $syncScript
    echo "# Usage: " >> $syncScript
    echo "#     bash moveDCStoLocalDB.sh idb_to_ldb.json" >> $syncScript
    echo -e "\n" >> $syncScript
fi


value="$(cat $MODULECFG | jq --arg a $module_id '.modules."\($a)"')"
if [ "$value" = "null" ]; then
    echo "[ error ][so] No module ID \"$module_id\" found under the entry \"modules\" in $MODULECFG."
    echo "[ info  ][so] Only the following module IDs are currently defined in the config file:"

    jq -r '.modules | keys[] as $k | "\($k) \(.[$k])"' $MODULECFG |
    while IFS=" " read -r -a value; do
        key=${value[0]}
        echo "-   $key"
    done

    echo "[ info  ][so] Please add yours (or rename one of the above) and try again."
    exit
fi

nChips="$(jq -j --arg b ${module_id} '.modules."\($b)".chips | length' $MODULECFG)"
cfgdir=$SOBASE/configs/rd53a/$module_id
CNCT_FILE=$cfgdir/connectivity.json

#-c option
if [ $createConfig == 1 ] ; then
    rm -rf $cfgdir
    echo "[ info ][so] Creating configs for module \"$module_id\""

    cmd="$YARRBASE/bin/dbAccessor -D -n $module_id -p $cfgdir"
    $cmd
    retval=$?
    if [ $retval -ne 0 ]; then
        if [ "$qcOpt" = "-Q" ]; then
            echo "[ info ][so] Please use localdb's viewer to register the module from iTkPD"
            exit
        else
            rm -rf $cfgdir

            echo "[so] Do you want to register \"$module_id\" in localDB?"
            unset answer
            while [ -z "${answer}" ]; do
                read -p "[y/n]: " answer
            done
            if [[ ${answer} == "y" ]]; then

                echo "------------------------"

                echo -e "\n[ info ][so] Site information (\"$LDBSITCFG\")"
                cat $LDBSITCFG | jq .

                echo -e "\n[ info ][so] User information (\"$LDBUSRCFG\")"
                cat $LDBUSRCFG | jq .

                echo ""

                # Generate module information file
                # > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > 
                modRegFile=$SOBASE/configs/ldb_module_registration.json
                touch $modRegFile

                echo "{}" > $modRegFile

                chipType=$(jq -j --arg a $module_id '.modules."\($a)".chipType' $MODULECFG)

                jq --arg a $module_id '."module" = {"serialNumber": "\($a)", "componentType": "Module"}' $modRegFile > sotmp && mv sotmp $modRegFile
                jq --arg a $chipType '."chipType" = "\($a)"' $modRegFile > sotmp && mv sotmp $modRegFile
                jq '."chips" = []' $modRegFile > sotmp && mv sotmp $modRegFile

                for (( i=0; i<$nChips; i++ ))
                do
                    chipId="$(jq -j --argjson a ${i} --arg b ${module_id} '.modules."\($b)".chips[$a].Parameter.ChipId' $MODULECFG)"
                    chipName="$(jq -j --argjson a ${i} --arg b ${module_id} '.modules."\($b)".chips[$a].connectivity.name' $MODULECFG)"

                    jq --argjson a ${i} --arg v $chipName '.chips[$a].serialNumber = "\($v)"' $modRegFile > sotmp && mv sotmp $modRegFile
                    jq --argjson a ${i}  '.chips[$a].componentType = "Front-end Chip"' $modRegFile > sotmp && mv sotmp $modRegFile
                    jq --argjson a ${i} --argjson b ${chipId}  '.chips[$a].chipId = $b' $modRegFile > sotmp && mv sotmp $modRegFile
                done

                echo "[ info ][so] Module registration file, (autogenerated from \"$MODULECFG\")"
                cat $modRegFile | jq .
                # < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < 


                echo -e "\n[ info ][so] Please make sure that these files (and the module registration file in particular) are correct before proceeding."
                echo "------------------------"
                
                echo "Are you ok with the settings above?"
                unset answer
                while [ -z "${answer}" ]; do
                    read -p "[y/n]: " answer
                done
                if [[ ${answer} == "y" ]]; then
                    retval=$($YARRBASE/bin/dbAccessor -C -c $modRegFile -u $LDBUSRCFG -i $LDBSITCFG)
                else
                    echo "[ info ][so] ok! Make the appropriate changes and come again"
                    exit
                fi
                #Retrieve cfg files from localdb
                cmd="$YARRBASE/bin/dbAccessor -D -n $module_id -p $cfgdir"
                $cmd
            else
                echo "Do you want to create temporary config files locally for \"$module_id\"?"

                unset answer
                while [ -z "${answer}" ]; do
                    read -p "[y/n]: " answer
                done
                if [[ ${answer} != "y" ]]; then
                    echo "[ info ][so] Oh, well, ok... Sorry, I somehow need those config files..."
                    exit
                fi
                conntmp="/tmp/$RANDOM"
                conn="{\"chipType\":\"$chiptype\",\"chips\":[]}"
                chiptype=$(jq -j --arg a $module_id '.modules."\($a)".chipType' $MODULECFG)

                if [ "$chiptype" != "null" ]; then
                    conn=$(echo $conn | jq ".chipType |= .+\"$chiptype\"")
                fi
                cnt=0
                while [ $cnt -lt $nChips ]; do

                    # "name" in connectivity.json should to match the actual chipName
                    chipName=$(jq -j --arg a $module_id --argjson b $cnt '.modules."\($a)".chips[$b].serial_number' $MODULECFG)
                    if [ "$chipName" != "null" ]; then
                        conn=$(echo $conn | jq ".chips[$cnt] |= .+{\"name\":\"$chipName\"}")
                    fi

                    # these parameters will be modified later
                    conn=$(echo $conn | jq ".chips[$cnt] |= .+{\"tx\":0}")
                    conn=$(echo $conn | jq ".chips[$cnt] |= .+{\"rx\":0}")
                    conn=$(echo $conn | jq ".chips[$cnt] |= .+{\"enable\":1}")
                    conn=$(echo $conn | jq ".chips[$cnt] |= .+{\"locked\":0}")

                    cnt=$(( cnt + 1 ))

                done
                echo $conn | jq . > $conntmp

                cmd="$YARRBASE/bin/dbAccessor -D -c $conntmp -p $cfgdir"
                $cmd
		retval=$?
                rm $conntmp
                if [ $retval -ne 0 ]; then
                    exit
                fi
            fi
        fi
    fi
    

    # Generate chip cfg files
    for (( i=0; i<$nChips; i++ ))
    do
        echo ""
        echo "------------------------"
        # create default chipConfig.json
        chipName="$(jq -j --argjson a ${i} --arg b ${module_id} '.modules."\($b)".chips[$a].connectivity.name' $MODULECFG)"
        echo -e "[ info ][so] Creating config file for chip \"$(green $chipName)\": $cfgdir/$chipName.json"
        $YARRBASE/bin/createConfig -t RD53A -n $chipName -o $cfgdir/$chipName.json #> /dev/null
    done


    # Generate controller cfg file
    CRTL_FILE=$cfgdir/controller.json
    JudgeCONNTP="$(jq --arg b ${module_id} '.modules."\($b)".controllerType' $MODULECFG)"
    if [ $JudgeCONNTP = "null" ]; then
	echo -e "$(red [\ error\ ][so]) Couldn't find the $(darkblue controllerType\ key) in $(darkblue $MODULECFG). Please add it! You can check how to write in $(darkblue $MODULECFG) at \"https://gitlab.cern.ch/YARR/utilities/scan-operator/-/blob/master/configs/so_modules.json\" ."
	exit
    fi
    ctrlType="$(jq -j --argjson a ${i} --arg b ${module_id} '.modules."\($b)".controllerType' $MODULECFG)"
    if [ ! -e $YARRBASE/configs/controller/specCfg-rd53a-${ctrlType}.json ]; then
	echo -e "$(red [\ error\ ][so]) Couldn't find $(darkblue \"$YARRBASE/configs/controller/specCfg-rd53a-${ctrlType}.json\")! Could you check whether the file exists?"
	exit
    fi
    if [ -e $YARRBASE/configs/controller/specCfg-rd53a-${ctrlType}.json ]; then
	cp $YARRBASE/configs/controller/specCfg-rd53a-${ctrlType}.json $CRTL_FILE 
	echo -e "[ info ][so] Generating $(darkblue \"controller.json\") with $(darkblue \"$YARRBASE/configs/controller/specCfg-rd53a-${ctrlType}.json\")"
	specNum=$(jq -j --arg b ${module_id} '.modules."\($b)".specNum' $MODULECFG)
	if [ "$specNum" != "null" ]; then
            jq --argjson v $specNum '.ctrlCfg.cfg.specNum = $v' $CRTL_FILE > currentValue && mv currentValue $CRTL_FILE
	fi
    fi
fi


# Check that cfg files are there
if [[ ! -d "$cfgdir" ]]
then
    echo -e "[ error ][so] Couldn't find config files for module \"$module_id\".\n"\
             "             Please run the S.O. again with '-c' to create them"
    exit
fi


echo ""
# Modify config files according to config.json
if [ $skipCfgCheck != 1 ] ; then
    for (( i=0; i<$nChips; i++ ))
    do

        # Chip config file 
        chipName="$(jq -j --argjson a ${i} --arg b ${module_id} '.modules."\($b)".chips[$a].connectivity.name' $MODULECFG)"
        JSON_FILE=$cfgdir/$chipName.json

        echo -e "[ info ][so] Looking for cfg updates in chip $(( i+1 ))'s configs..."
        jq -r --argjson a ${i} --arg b ${module_id} '.modules."\($b)".chips[$a].GlobalConfig | keys[] as $k | "\($k) \(.[$k])"' $MODULECFG |
        while IFS=" " read -r -a value; do
            key=${value[0]}
            val=${value[1]}

            currentValue=$(jq --arg k ${key} '.RD53A.GlobalConfig."\($k)"' $JSON_FILE)

            if [ "$currentValue" == "null" ];
            then
                jq --argjson v $val --arg k $key '.RD53A.GlobalConfig."\($k)" = $v' $JSON_FILE > currentValue && mv currentValue $JSON_FILE
                echo -e "[ warn ][so] Added key $(darkblue \"$key\") with value $(darkblue \"$val\") in the chip config file (GlobalConfig section)"

            elif [ "$currentValue" != $val ];
            then
                jq --argjson v $val --arg k $key '.RD53A.GlobalConfig."\($k)" = $v' $JSON_FILE > currentValue && mv currentValue $JSON_FILE
                echo -e "[ info ][so] Changed $(darkblue \"$key\") from $(darkblue \"$currentValue\") to $(darkblue \"$val\") in the chip config file (GlobalConfig section)"
            fi
        done

        # Parameter section
        jq -r --argjson a ${i} --arg b ${module_id} '.modules."\($b)".chips[$a].Parameter | keys[] as $k | "\($k) \(.[$k])"' $MODULECFG |
        while IFS=" " read -r -a value; do
            key=${value[0]}
            val=${value[1]}

            currentValue=$(jq -r --arg k ${key} '.RD53A.Parameter."\($k)"' $JSON_FILE)

            if [ "$currentValue" == "null" ];
            then
                jq --argjson v $val --arg k $key '.RD53A.Parameter."\($k)" = $v' $JSON_FILE > currentValue && mv currentValue $JSON_FILE
                echo -e "[ warn ][so] Added key $(darkblue \"$key\") with value $(darkblue \"$val\") in the chip config file (Parameter section)"

            elif [ "$currentValue" != "$val" ];
            then
                if ! [[ "$val" =~ ^[0-9]+$ ]] ; then
                    jq --arg v $val --arg k $key '.RD53A.Parameter."\($k)" = "\($v)"' $JSON_FILE > currentValue && mv currentValue $JSON_FILE

                else
                    jq --argjson v $val --arg k $key '.RD53A.Parameter."\($k)" = $v' $JSON_FILE > currentValue && mv currentValue $JSON_FILE
                fi

                echo -e "[ info ][so] Changed $(darkblue \"$key\") from $(darkblue \"$currentValue\") to $(darkblue \"$val\") in the chip config file (Parameter section)"
            fi
        done
        # Connectivity file:
        jq -e --argjson a ${i} --arg v $chipName '.chips[$a].name = "\($v)"' $CNCT_FILE > currentValue && mv currentValue $CNCT_FILE
        jq -e --argjson a ${i} --arg v $JSON_FILE '.chips[$a].config = "\($v)"' $CNCT_FILE > currentValue && mv currentValue $CNCT_FILE
        jq -r --argjson a ${i} --arg b ${module_id} '.modules."\($b)".chips[$a].connectivity | keys[] as $k | "\($k) \(.[$k])"' $MODULECFG |
        while IFS=" " read -r -a value; do
            key=${value[0]}
            val=${value[1]}

            currentValue=$(jq -r --argjson a ${i} --arg k ${key} '.chips[$a]."\($k)"' $CNCT_FILE)

            if [ "$currentValue" == "null" ];
            then
                jq --argjson a ${i} --argjson v $val --arg k $key '.chips[$a]."\($k)" = $v' $CNCT_FILE > currentValue && mv currentValue $CNCT_FILE
                echo -e "[ warn ][so] Added key $(darkblue \"$key\") with value $(darkblue \"$val\") in the connectivity file (chip $i)"

            elif [ "$currentValue" != "$val" ];
            then
                if ! [[ "$val" =~ ^[0-9]+$ ]] ; then
                    jq --argjson a ${i} --arg v $val --arg k $key '.chips[$a]."\($k)" = "\($v)"' $CNCT_FILE > currentValue && mv currentValue $CNCT_FILE
                else
                    jq --argjson a ${i} --argjson v $val --arg k $key '.chips[$a]."\($k)" = $v' $CNCT_FILE > currentValue && mv currentValue $CNCT_FILE
                fi

                echo -e "[ info ][so] Changed $(darkblue \"$key\") from $(darkblue \"$currentValue\") to $(darkblue \"$val\") in the connectivity file (chip $i)"
            fi
        done


        # Create an entry with the module serial number (to associate the config
        # files to in localdb)
        jq '. += {"module"}' $CNCT_FILE > sotmp && mv sotmp $CNCT_FILE
        jq --arg a ${module_id} '.module += {"serialNumber": "\($a)"}' $CNCT_FILE > sotmp && mv sotmp $CNCT_FILE
        jq '.module += {"componentType": "module"}' $CNCT_FILE > sotmp && mv sotmp $CNCT_FILE
        echo ""
    done
fi


echo "[ info ][so] Current connectivity config:"
echo "------------------------"
cat $cfgdir/connectivity.json | jq .
echo "------------------------"


if [ $justConfigure == 1 ] ;then
    echo -e "[ info ][so] Configuration updated."
    exit 0
fi

if [ $dcsOnOff == 1 ] ;then
    echo -e "\n[ info ][so] Turning ON the PS..."

    nChannels="$(jq -j '.channels_to_be_used_by_the_scanoperator | length' $PSCFG)"
    for (( i=0; i<$nChannels; i++ ))
    do
        chn="$(jq -j ".channels_to_be_used_by_the_scanoperator[$i].name" $PSCFG)"
        python3 $SOBASE/libDCS/psOperator.py -e $PSCFG -c $chn -d $INFLUXDBDCSCFG power-on
        if [ $? -ne 0 ]; then exit 1; fi
    done
fi


echo -e "\n[ info ][so] Running the sequence of scans"


nScans="$(jq -j '.scan_list | length' $SCANCFG)"
for i in $(seq 1 $nScans); do

    if [[ "$dcsMonitor" == "1" ]]; then
        waitForItToFinish "read_dcs_backgr"
        dcsOpt="-d"
    fi

    if [[ $envMonitor == 1 ]]; then
        # check if T and H are within the working interval before every scan
        python3 $SOBASE/libDCS/getFromInfluxDB.py -j $INFLUXDBENVCFG
        retval=$?
    if [ $retval -ne 0 ]; then
        wrapAndExit
        break
    fi
    fi

    obj="$(jq -j  --argjson a ${i} '.scan_list[$a-1]' $SCANCFG)"
    obj="$(echo -e "${obj}" | tr -d '[:space:]')"
    
    comm="$SOBASE/scanLauncher.sh -m ${module_id} -s $obj ${dcsOpt} ${localdbOpt} ${qcOpt} ${syncOption}"
    echo "$comm"
    echo -e "[ info ] |\n[ info ] | - $(echo $obj | jq -r '.[0]')\n[ info ] |"
    if [ $quietMode -gt 0 ];
    then
        output=$($comm 2>&1 | awk '/ Mean /{print $0}')
        retval=$?
        if [ "$output" != "" ];then
            while IFS= read -r line ; do echo -e "[ info ] |     -> ${line##*\] }"; done <<< "$output"
        else
            echo -e "[ info ] |     -> done"
        fi
    else
        $comm
        retval=$?
    fi
    if [ $retval -ne 0 ]; then
        wrapAndExit
        break
    fi
done



if [ $dcsOnOff == 1 ]; then
    echo -e "\n[ info ][so] Turning OFF the PS..."

    nChannels="$(jq -j '.channels_to_be_used_by_the_scanoperator | length' $PSCFG)"
    for (( i=0; i<$nChannels; i++ ))
    do
        chn="$(jq -j ".channels_to_be_used_by_the_scanoperator[$i].name" $PSCFG)"
        python3 $SOBASE/libDCS/psOperator.py -e $PSCFG -c $chn -d $INFLUXDBDCSCFG power-off
        if [ $? -ne 0 ]; then exit 1; fi
    done
fi

if [[ "$dcsMonitor" != "" ]]; then
    #wait for the dcs monitor script to finish
    waitForItToFinish "read_dcs_backgr"
fi

if [[ "$syncOption" == "" ]] && [[ "$localdbOpt" != "" ]]
then

cat << EOF
[ info ][so] If you want to upload to localDB the DCS data associated
             to all the scans you have just run (assuming you have it 
             stored in InfluxDB), you can do it in one command:

                 bash $SOBASE/data/$SO_RUNNUMBER/moveDCStoLocalDB.sh path/to/idb_to_ldb.json
EOF

fi

elp=$(echo "$(date +%s.%N) - $START_SO" | bc)
echo -ne "total_so     $elp\n" >> $TIMEFILE

echo -e "[ info ][so] All done"

